package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.SceneLoader;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ENUMs.AccountType;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.DatabaseException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextField;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.VBox;

import java.net.URL;
import java.util.ResourceBundle;

public class CreateAccountController implements Initializable {
    @FXML
    private VBox parent;
    @FXML
    private RadioButton local;
    @FXML
    private ToggleGroup statistics;
    @FXML
    private TextField playerName, password;
    @FXML
    private Label errorMessage, passwordLabel;

    String selectedButton;
    SceneLoader sceneLoader = new SceneLoader();
    Persistence persistence = Persistence.getInstance();

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        getSelected();
    }

    private void getSelected() {
        statistics.selectedToggleProperty().addListener((observableValue, toggle, t1) -> {
            if (statistics.getSelectedToggle() != null) {
                RadioButton selectedToggle = (RadioButton) statistics.getSelectedToggle();
                selectedButton = selectedToggle.getText();
                System.out.println(selectedButton);
            }
            if (local.isSelected()) {
                password.setEditable(false);
                password.setText("");
                passwordLabel.setStyle("-fx-text-fill: rgba(0, 0, 0, 0.3)");
                password.setStyle("-fx-border-color: rgba(0, 0, 0, 0.3)");
            } else {
                passwordLabel.setStyle("-fx-text-fill: black");
                password.setEditable(true);
                password.setStyle("-fx-border-color: black");
            }
        });
    }

    private void switchScene(String name) {
        parent.getChildren().clear();
        parent.getChildren().add(sceneLoader.loadScene(name));
    }

    @FXML
    private void create() {
        AccountType accountType;
        if (selectedButton.equals("Local")) {
            accountType = AccountType.LOCAL;
        } else {
            accountType = AccountType.ONLINE;
        }
        try {
            persistence.createAccount(playerName.getText(), password.getText(), accountType);
            persistence.resetPlayerStatistics();
            persistence.loadPlayerStatistics();
            switchScene("MainMenu");
        } catch (DatabaseException e) {
            errorMessage.setText(e.getMessage());
        }
    }

    @FXML
    private void back() {
        switchScene("PlayerCreateScene");
    }
}
