package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.MusicPlayerSingleton;
import de.hdm_stuttgart.battlearena.Controller.Utilities.SceneLoader;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.MapData;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Objects;
import java.util.ResourceBundle;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

public class MapSelectionController implements Initializable {
    @FXML
    private VBox boxBox, parent;
    @FXML
    private Text selectedMapText;

    private int selectedMap;

    private final SceneLoader sceneLoader = new SceneLoader();

    private final ArrayList<MapData> coreMaps = Persistence.getInstance().getCoreMaps();

    private static Logger log = LogManager.getLogger(MapSelectionController.class);

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        ToggleGroup group = new ToggleGroup();

        for (MapData coreMap : coreMaps) {
            ToggleButton toggleButton = new ToggleButton(coreMap.getMapName());
            toggleButton.setToggleGroup(group);
            boxBox.getChildren().add(toggleButton);
        }

        group.selectedToggleProperty().addListener((observableValue, oldToggle, newToggle) -> {
            if (group.getSelectedToggle() != null) {
                ToggleButton selected = (ToggleButton) group.getSelectedToggle();
                selectedMap = boxBox.getChildren().indexOf(selected);
                System.out.println(selectedMap);
                String value = selected.getText();
                selectedMapText.setText(value);
                System.out.println(value);
            }
        });
    }

    @FXML
    private void gameScene() {
        try {
            RuntimeInfo.getInstance().setGameMap(coreMaps.get(selectedMap));
            MusicPlayerSingleton.getInstance().getMediaPlayer().dispose();
            parent.getScene().setRoot(FXMLLoader.load(Objects.requireNonNull(getClass().getResource("/fxml/GameScene.fxml"))));
        } catch (IOException e) {
            log.info(e.getCause());
            throw new RuntimeException();
        }
    }

    @FXML
    private void backButton() {
        parent.getChildren().clear();
        parent.getChildren().add(sceneLoader.loadScene("SkinSelection"));
    }
}
