package de.hdm_stuttgart.battlearena.Model.DataStorage.Classes;

import de.hdm_stuttgart.battlearena.Controller.Enum.GameMode;
import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.ParserException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.HashGenerator;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.Parser;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityClass;
import de.hdm_stuttgart.battlearena.Model.Entity.IEntity;
import de.hdm_stuttgart.battlearena.Model.Map.Biom;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;

public class RuntimeInfo {

    private static final Logger log = LogManager.getLogger(RuntimeInfo.class);
    private static final RuntimeInfo runtimeInfoSingleton = new RuntimeInfo();
    private String mapDataGame;
    private MapData mapGame;
    private boolean offlineMode;  //if Account-Type is online but no SQL connection: start game without stats tracking
    private int musicVolume = 50;
    private int sfxVolume = 50;
    private GameState gameState = GameState.NONE; //Default value: MENU
    private GameMode gameMode = GameMode.LOCAL; //Default value: LOCAL

    //Default value: HUMAN
    private EntityClass playerOneClass = EntityClass.HUMAN;
    private EntityClass playerTwoClass = EntityClass.HUMAN;

    private Biom mapBiom = Biom.BOMBERMAN; //Default value: GRASS

    private List<IEntity> gameplayObjects = new ArrayList<>();

    //Stats Tracking during Gameplay
    private int kills = 0;
    private int deaths = 0;
    private int blocksDestroyed = 0;
    private int gameTime = 0;
    private boolean gameWon = true;


    private RuntimeInfo() {
    }


    public static RuntimeInfo getInstance() {
        return runtimeInfoSingleton;
    }

    public MapData generateMap(String mapName, String mapData) throws ParserException {
        try {
            Parser.mapDataValid(mapData);
            Parser.mapNameValid(mapName);
            return new MapData(HashGenerator.hashAndHex(mapData), mapName, 18, 18, mapData);
        } catch (Exception e) {
            log.error(e);
            throw new ParserException(e.getMessage());
        }
    }

    public void setGameMap(MapData mapSelected) {    //Create Game Scene Dropdown list directly gives reference to selected map
        try {
            mapGame = mapSelected;
            mapDataGame = mapSelected.getMapData();
        } catch (Exception e) {
            log.error(e);
        }
    }

    public void resetGameStats() {
        kills = 0;
        deaths = 0;
        blocksDestroyed = 0;
        gameTime = 0;
        gameWon = true;
    }

    public GameState getGameState() {
        return gameState;
    }

    public void setGameState(GameState gameState) {
        this.gameState = gameState;
    }

    public GameMode getGameMode() {
        return gameMode;
    }

    public void setGameMode(GameMode gameMode) {
        this.gameMode = gameMode;
    }

    public int getMusicVolume() {
        return musicVolume;
    }

    public void setMusicVolume(int musicVolume) {
        this.musicVolume = musicVolume;
    }

    public int getSfxVolume() {
        return sfxVolume;
    }

    public void setSfxVolume(int sfxVolume) {
        this.sfxVolume = sfxVolume;
    }

    public EntityClass getPlayerOneClass() {
        return playerOneClass;
    }

    public void setPlayerOneClass(EntityClass playerOneClass) {
        this.playerOneClass = playerOneClass;
    }

    public EntityClass getPlayerTwoClass() {
        return playerTwoClass;
    }

    public void setPlayerTwoClass(EntityClass playerTwoClass) {
        this.playerTwoClass = playerTwoClass;
    }

    public Biom getMapBiom() {
        return mapBiom;
    }

    public void setMapBiom(Biom mapBiom) {
        this.mapBiom = mapBiom;
    }

    public List<IEntity> getGameplayObjects() {
        return gameplayObjects;
    }

    public void setGameplayObjects(List<IEntity> gameplayObjects) {
        this.gameplayObjects = gameplayObjects;
    }

    public String getMapDataGame() {
        return mapDataGame;
    }

    public int getKills() {
        return kills;
    }

    public void setKills(int kills) {
        this.kills = kills;
    }

    public void addKill() {
        kills = kills + 1;
    }

    public int getDeaths() {
        return deaths;
    }

    public void setDeaths(int deaths) {
        this.deaths = deaths;
    }

    public void addDeath() {
        deaths = deaths + 1;
    }

    public int getBlocksDestroyed() {
        return blocksDestroyed;
    }

    public void setBlocksDestroyed(int blocksDestroyed) {
        this.blocksDestroyed = blocksDestroyed;
    }

    public void addBlockDestroyed() {
        blocksDestroyed = blocksDestroyed + 1;
    }

    public int getGameTime() {
        return gameTime;
    }

    public void setGameTime(int gameTime) {
        this.gameTime = gameTime;
    }

    public void addGameTime() {
        gameTime = gameTime + 1;
    }

    public boolean isGameWon() {
        return gameWon;
    }

    public void setGameWon(boolean gameWon) {
        this.gameWon = gameWon;
    }

}
