package de.hdm_stuttgart.battlearena.Model.Sound;


import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.AppSettings;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import javafx.application.Platform;
import javafx.fxml.Initializable;
import javafx.scene.media.Media;
import javafx.scene.media.MediaPlayer;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;


import java.io.IOException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.ResourceBundle;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class MusicPlayer{



    private static final Logger log = LogManager.getLogger(MusicPlayer.class);

    private static final MusicPlayer  musicPlayerSingleton = new MusicPlayer();

    private AppSettings settings = Persistence.getInstance().getSettings();
    private MediaPlayer mediaPlayer;
    private GameState currentGameState;
    private SoundFileManager fileManager;

    public MusicPlayer(){}




    public void startGameStateMonitoring(){
        ScheduledExecutorService scheduler = Executors.newSingleThreadScheduledExecutor();
        scheduler.scheduleAtFixedRate(()->{
           GameState newGameState = RuntimeInfo.getInstance().getGameState();
           if(currentGameState != newGameState){
               currentGameState = newGameState;
               Platform.runLater(this::updateMusic);
           }
        },0,1, TimeUnit.SECONDS);

    }

    private void updateMusic(){
        stopMusic();
        playRandomMusic();
    }

    private void playRandomMusic() {
        String path = currentGameState.getPath();
        if(!path.isEmpty()) {
            Path pathTypeOfMusic = Paths.get(path);

            List<Path> musicPaths;
            try {
                musicPaths = fileManager.getFilePathsFromResources(pathTypeOfMusic);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }

            String randomPath = fileManager.getRandomFilePath(musicPaths);
            String resource = fileManager.convertPathToResourcePath(randomPath);

            Thread thread = new Thread(() -> {
                Media media = new Media(getClass().getResource(resource).toExternalForm());
                mediaPlayer = new MediaPlayer(media);
                mediaPlayer.setCycleCount(MediaPlayer.INDEFINITE);
                mediaPlayer.setVolume((double) settings.getMusicVolume() / 100);
                mediaPlayer.play();
                log.info("Music is playing");
            });
            thread.setDaemon(true);
            thread.start();
        }
    }


    public void stopMusic(){
        if (mediaPlayer != null) {
            mediaPlayer.dispose(); // Dispose of the MediaPlayer instance to ensure stoping
        }
    }

    private void setMediaPlayer(MediaPlayer mediaPlayer){
        this.mediaPlayer = mediaPlayer;
    }

    public void setVolume(int volume){
        settings.setMusicVolume(volume);
        mediaPlayer.setVolume((double) volume / 100);
    }

    public int getVolume(){
        return settings.getMusicVolume();
    }

    public MusicPlayer getInstance(){return musicPlayerSingleton;}

}
